<?php
abstract class Akrabat_Config_Abstract implements ArrayAccess, IteratorAggregate
{
    protected $_config;
    protected $_allowModifications;

    /**
     * Load the section $section from the config file $filename.
     *
     * If any keys with $section are called "extends", then the section
     * pointed to by the "extends" is then included into the properties.
     * Note that the keys in $section will override any keys of the same
     * name in the sections that have been included via "extends".
     *
     * If any key includes a ".", then this will act as a separator to
     * create a sub-property.
     *
     * Only allow modifications after construction if $allowModifications
     * is set to true.
     *
     * example ini file:
     *      [all]
     *      db.connection = database
     *      hostname = live
     *
     *      [staging]
     *      include = all
     *      hostname = staging
     *
     * after calling $config = new Akrabat_Config($file, 'staging'); then
     *      $config->hostname = staging
     *      $config->db->connection = database
     *
     *
     * @param string $filename
     * @param string $section
     * @param boolean $allowModifications
     */
    function __construct($filename, $section, $allowModifications=false)
    {
        if(!isset($section))
        {
            throw new Exception("Section is not set!");
        }
        $this->_allowModifications = $allowModifications;
        $this->_config = array();
        $this->load($filename, $section);
    }

    /**
     * Load a config file, overwriting any duplicate keys.
     * If $section is null, then the entire file is loaded.
     *
     * @param string $filename
     * @param string $section
     */
    abstract public function load($filename, $section);

    /**
     * @param string $name
     * @param mixed $default
     * @return mixed
     */
    public function get($name, $default=null)
    {
        $result = $default;
        if (isset($this->_config->$name)) {
            $result = $this->_config->$name;
        }
        return $result;
    }

    /**
     * magic function so that $obj->value will work.
     *
     * @param string $name
     * @return mixed
     */
    public function __get($name)
    {
        return $this->get($name);
    }

    /**
     * Only allow setting of a property if allowModifications
     * is true. Otherwise, throw an exception.
     *
     * @param string $name
     * @param mixed $value
     */
    public function __set($name, $value)
    {
        if($this->_allowModifications) {
            $this->_config->$name = $value;
        } else {
            throw new Exception('Akrabat_Config is read only!');
        }
    }

    /**
     * Does the config key $name exist?
     *
     * @param string $name
     * @return boolean
     */
    public function exists($name)
    {
        return isset($this->_config->$name);
    }

    /**
     * Defined by ArrayAccess interface
     * Set a value given it's key e.g. $A['title'] = 'foo';
     * @param mixed key (string or integer)
     * @param mixed value
     * @return void
     */
    function offsetSet($key, $value) {
        if ( array_key_exists($key,get_object_vars($this->_config)) ) {
            $this->_config->{$key} = $value;
        }
    }

    /**
     * Defined by ArrayAccess interface
     * Return a value given it's key e.g. echo $A['title'];
     * @param mixed key (string or integer)
     * @return mixed value
     */
    function offsetGet($key) {
        if ( array_key_exists($key,get_object_vars($this->_config)) ) {
            return $this->_config->{$key};
        }
    }

    /**
     * Defined by ArrayAccess interface
     * Unset a value by it's key e.g. unset($A['title']);
     * @param mixed key (string or integer)
     * @return void
     */
    function offsetUnset($key) {
        if ( array_key_exists($key,get_object_vars($this->_config)) ) {
            unset($this->_config->{$key});
        }
    }

    /**
     * Defined by ArrayAccess interface
     * Check value exists, given it's key e.g. isset($A['title'])
     * @param mixed key (string or integer)
     * @return boolean
     */
    function offsetExists($offset) {
        return array_key_exists($offset,get_object_vars($this->_config));
    }

    /**
     * Defined by IteratorAggregate interface
     * Returns an iterator for for this object, for use with foreach
     * @return ArrayIterator
     */
     function getIterator() {
       return new ArrayIterator($this->_config);
     }
}
?>